import {DEFAULT_HOSTNAME, FAIL, JS_BRIDGE, SUCCESS} from './const';
import {extend, isFunction, isInApp, isIOS, isObject, isString} from "./utils";
import {__paramInvoke__} from "./__handle__/__handle_utils__";
import __bus__ from "./__handle__/__bus__";

/**
 * 设置 config 方法
 * @param setting
 */
export const config = (setting = {}) => {
	_ready(function () {
		// 非sdk环境 或 设置config成功了，则立即返回回调
		if(!isInApp() || !!__bus__.global.device){
			if (isFunction(setting[SUCCESS])){
				setting[SUCCESS].call(null, __bus__.device);
			}
			if (isFunction(__bus__.global.readyCallback)) {
				__bus__.global.readyCallback.call(null, __bus__.device);
			}
			return;
		}
		let defaultSetting = {
			hostname: ""
		};
		if(location.href.indexOf('file://') !== -1){ //本地页面
			defaultSetting.hostname = DEFAULT_HOSTNAME;
		}
		const _setting = extend(defaultSetting, __paramInvoke__(setting));
		_setting.success = function (device) {
			__bus__.setDevice(device);
			if (isFunction(setting[SUCCESS])){
				setting[SUCCESS].call(null, device)
			}
		};
		_setting.fail = function (err) {
			log("config with error => " + JSON.stringify(err))
			__bus__.setDevice({});
			if (isFunction(setting[FAIL])){
				setting[FAIL].call(null, err)
			}
		};
		setTimeout(function () {
			callHandler("config", _setting);
		}, 0);
		// fix android bug, set config null
		__bus__.global.st = setInterval(function () {
			if (!!__bus__.global.device){
				clearInterval(__bus__.global.st)
			}else {
				log("set config timeout...");
				callHandler("config", _setting);
			}
		}, 800)
	});
};

/**
 * 注册onReady方法
 * @param callback
 */
export const onReady = (callback = ()=>{}) => {
	__bus__.global.readyCallback = callback;
	register('onReady', data => {
		// __bus__.device = __bus__.global.device  = data
		__bus__.setDevice(data);
		// if (__bus__.global.sccbaJsApi){
		// 	__bus__.global.sccbaJsApi["device"] = data;
		// }
		callback.call(null, data);
	});
};

/**
 * 注册onError方法
 * @param callback
 */
export const onError = (callback = () => {}) => {
	register('onError', () => {
		callback.call(null);
	});
}

/**
 * 绑定JS API
 * @param method
 * @param param
 */
export function callHandler(method, param) {
	return new Call(method, param).callHandler();
}

/**
 * js注册事件监听，由客户端触发
 * @param eventName 事件名
 * @param callback 接收回调函数
 * @returns {*}
 *  * Example:
 *  bridge.registerHandler("showAlert", function(data) { alert(data) })
 bridge.registerHandler("getCurrentPageUrl", function(data, responseCallback) {
	    responseCallback(document.location.toString())
    });
 	sccbaJsApi.register("onResume', function(arg){
 		console.log("native send message: "+ arg);
 	})
	 sccbaJsApi.register("configVoiceKeywords', function(arg, nativeCallback){
			console.log("native send message: "+ arg);
			nativeCallback(["a", "b"]); //给客户端回调数据
		})
 */
export function register(eventName, callback) {
	_ready(() => {
		let entry = __bus__.global["eventEntry"] || {};
		entry[eventName] = callback;
		if(__bus__.global[JS_BRIDGE]){
			return __bus__.global[JS_BRIDGE].registerHandler(eventName, (data, nativeCallback) => {
				let responseCallback = entry[eventName];
				if (isFunction(responseCallback)) {
					responseCallback.call(null,data, nativeCallback);
				}
			});
		}else {
			log("js bridge 未初始化，无法注册监听");
			return callback && callback();
		}

	});
}

export  {register as addEventListener};

/**
 * 移除监听
 * @param eventName
 */
export const removeRegister = eventName => {
	let entry = __bus__.global["eventEntry"] || {};
	if (entry[eventName]) {
		delete entry[eventName];
	}
};
export  {removeRegister as removeEventListener};
/**
 * 客户端打印日志信息
 * @param param
 */
export const log = (param) => {
	let _param;
	if (isString(param)) {
		_param = { message: param };
	} else if (isObject(param)) {
		_param = param;
	} else {
		_param = {};
	}
	if (window[JS_BRIDGE]) {
		new Call("log", _param).callHandler();
	} else {
		console.log("[JS-BRIDGE]: " + _param.message);
	}
};

/**
 * js api 校验
 * @param opt
 */
export const checkJsApi = opt => {
	callHandler('checkJsApi', opt);
};

export const sccbaJSBridgeReady = _ready;

//////////////////////////////////////////////////////
/**
 * Call 对象
 * @param method
 * @param param
 * @constructor
 */
class Call {
	constructor(method = "", param = {}){
		this.method = method;
		this.param = param;
	}
	callHandler(){
		_call(this.method, __paramInvoke__(this.param), result => {
			if (isObject(result)) {
				if (
					result[SUCCESS] &&
					isFunction(this.param[SUCCESS])
				) {
					return this.param[SUCCESS](result[SUCCESS]);
				} else if (
					isObject(result[FAIL]) &&
					isFunction(this.param[FAIL])
				) {
					return this.param[FAIL](result[FAIL]);
				}
			}
		});
	}
}

/**
 * 调用 js bridge callHandler方法
 * @private
 */
function _call() {
	let args = [].slice.call(arguments);
	if (window[JS_BRIDGE] && window[JS_BRIDGE].callHandler) {
		//强制转为 name + object + function 形式的入参
		let name = args[0],
			opt = args[1] || {},
			cb = args[2];
		if (!isString(name)) {
			log({ message: 'Api Name error：', name });
			return;
		}
		if (cb === undefined && isFunction(opt)) {
			cb = opt;
			opt = {};
		}
		if (!isObject(opt)) {
			log({ message: `${name}: options is not object` });
			return;
		}
		let _callback = cb;
		cb = function(result) {
			_callback && _callback(result);
		};
		window[JS_BRIDGE].callHandler(name, opt, cb);
	} else {
		console.warn('js bridge not ready, call later');
	}
}

/**
 * js bridge ready
 * @param callback
 * @returns {*}
 * @private
 */
function _ready(callback) {
	if (isInApp()) {
		if (window[JS_BRIDGE]) {
			return callback(window[JS_BRIDGE]);
		}
		if (window.WVJBCallbacks) {
			return window.WVJBCallbacks.push(callback);
		}
		window.WVJBCallbacks = [callback];
		let WVJBIframe = document.createElement("iframe");
		WVJBIframe.style.display = "none";
		//这里最新IOS版是 https的scheme，真实环境下 需要判断iOS和Android，做下区分。
		WVJBIframe.src = isIOS()
			? "https://__bridge_loaded__"
			: "wvjbscheme://__BRIDGE_LOADED__";
		document.documentElement.appendChild(WVJBIframe);
		setTimeout(function() {
			document.documentElement.removeChild(WVJBIframe);
		}, 0);
	} else {
		if(document.isLoaded){
			return callback && callback({});
		}
		document.addEventListener(
			"DOMContentLoaded",
			function() {
				document.isLoaded = true
				callback && callback({});
			},
			false
		);
	}
}
