import { callHandler, register, removeRegister } from "./jsbridge";
import { FAIL, SUCCESS } from "./const";
import { extend, isEmptyObject, isInApp, isNumber } from "./utils";

/**
 * 获取地理位置
 * @param param
 * .eg
 *  sccbaJsApi.getLocation({
 *      success:function(){
 *
 *      }
 *  });
 */
export const getLocation = (param = {}) => {
	if (isInApp()) {
		callHandler("getLocation", param);
	} else {
		if (navigator.geolocation) {
			//浏览器支持获取地理位置
			let success = position => {
				param[SUCCESS] &&
					param[SUCCESS]({
						longitude: position.coords.longitude,
						latitude: position.coords.latitude
					});
			};
			let error = error => {
				param[FAIL] &&
					param[FAIL]({ errorCode: error.code, message: error.message });
			};
			navigator.geolocation.getCurrentPosition(success, error, {
				enableHighAccuracy: false,
				timeout: 10000,
				maximumAge: 10000
			});
		}
	}
};

/**
 * 获取昨天和现在的步数
 * @param opt
 */
export const getStepCount = (opt) => {
	opt.dateType = opt.dateType && (opt.dateType === "YESTERDAY" || opt.dateType === "NOW") ? opt.dateType : "NOW";
	callHandler("getStepCount", opt);
};

/**
 * 获取昨天步数
 * @param opt
 * @param success
 * @param fail
 */
export const getYesterdayStepCount = (success, fail) => {
	let opt = { dateType: "YESTERDAY" };
	opt.success = success;
	opt.fail = fail;
	getStepCount(opt);
};

/**
 * 获取今天现在的步数
 * @param opt
 * @param success
 * @param fail
 */
export const getNowStepCount = (success, fail) => {
	let opt = { dateType: "NOW" };
	opt.success = success;
	opt.fail = fail;
	getStepCount(opt);
};

/**
 * 调用系统发短信，打电话等的功能。
 * @param opt
 */
export const openURL = opt => {
	callHandler('openURL', opt);
};

/**
 * 打开外部APP
 * @param url
 * @param success
 * @param fail
 * @see openURL
 */
export const openApp = (url, success, fail) => {
	if (url.indexOf('openapp=') === -1) {
		url = 'openapp=' + url;
	}
	openURL({
		url: url,
		success: success,
		fail: fail
	});
};

/**
 * 复制文本
 * @param opt
 * @since v2.0.0
 */
export const copyText = opt => {
	callHandler("copyText", opt);
};

/**
 * 使用手机地图软件导航
 * @param opt
 * @since v2.0.0
 */
export const openMapAppNavigation = opt => {
	callHandler("openMapAppNavigation", opt);
};

/**
 * 选择手机通讯录联系人
 * @param opt
 * @since v2.0.0
 */
export const choosePhoneContacts = opt => {
	callHandler("choosePhoneContacts", opt);
}

/**
 * 调用平台设备的认证
 * @param opt
 * @since v2.0.0
 */
export const platformVerify = opt => {
	callHandler("platformVerify", opt);
};

/**
 * 获取当前网络状态
 * @param opt
 * @since v1.0.0
 */
export const getNetworkType = opt => {
	callHandler("getNetworkType", opt);
};

/**
 * 截屏保存到相册
 * @param opt
 * @since v1.0.0
 */
export const screenShots = (opt = {}) => {
	let defaultSetting = {
		startPosY: 0
	};
	let setting = {};
	if (!isEmptyObject(opt)) {
		setting = extend(defaultSetting, opt);
	}
	callHandler("screenShots", setting);
};

/**
 * 启用系统防截屏控制
 * @param opt
 * @since v1.0.0
 */
export const antiScreenShot = opt => {
	callHandler("antiScreenShot", opt);
};

/**
 * 取消启用防截屏功能
 * @param opt
 * @since v1.0.0
 */
export const clearScreenShot = opt => {
	callHandler("clearScreenShot", opt);
};

/**
 * 调节设备屏幕亮度
 * @param opt
 * @since v1.0.0
 */
export const setScreenBrightness = opt => {
	callHandler("setScreenBrightness", opt);
};


/**
 * 还原为用户原来的屏幕亮度
 * @param opt
 * @since v1.0.0
 */
export const clearScreenBrightness = opt => {
	callHandler("clearScreenBrightness", opt);
};


/**
 * 插件名称 getNetInfo 参数 无 得到Wifi数据
 * @param opt
 * @since v1.0.0
 */
export const getNetInfo = opt => {
	callHandler("getNetInfo", opt);
};

/**
 * 插件名称 getDeviceInfo 参数 无 得到设备信息
 * @param opt
 * @since v1.0.0
*/
export const getDeviceInfo = opt => {
	callHandler("getDeviceInfo", opt);
};

/**
 * 插件名称 scanQRCode 参数 无 扫描二维码
 * @param opt
 * @since v1.0.51
*/
export const scanQRCode = opt => {
	callHandler("scanQRCode", opt);
};

/**
 * 插件名称 openESSC 扫描电子社保卡插件
 * @param opt
 * @since v1.0.52
*/
export const openESSC = opt => {
	callHandler("openESSC", opt);
};

/**
 * 插件名称 closeESSC 关闭扫描电子社保卡插件
 * @param opt
 * @since v1.0.53
*/
export const closeESSC = opt => {
	callHandler("closeESSC", opt);
};
