import {extend, isInApp, isString} from "./utils";
import {callHandler,log} from "./jsbridge";
import __bus__ from "./__handle__/__bus__";
import {requestInterceptor} from './interceptors';
import {SUBMIT_KEY} from "./const";

export const request = _request;

/**
 * 便捷的网络请求方法
 * @param url       必选，xxx.do 无需拼完整url,会自动补全
 * @param opId      必选，reqHead 里的opId
 * @param transId   必选，reqHead 里的transId
 * @param reqData   必选，对应的reqData
 * @param success   可选，成功回调
 * @param fail      可选，失败回调
 * @param setting   可选，设置项，不传用默认设置
 */
export const simpleRequest = (url = "",
							  opId = "",
							  transId = "",
							  reqData = {},
							  success,
							  fail,
							  setting = {}) => {
	request({
		url: url,
		data: {
			reqHead: {
				opId: opId,
				transId: transId
			},
			reqData: reqData
		},
		success: success,
		fail: fail,
		setting: setting
	});
};

/**
 * 获取 submit key
 * @param url       必选，xxx.do 无需拼完整url,会自动补全
 * @param opId      必选，reqHead 里的opId
 * @param transId   必选，reqHead 里的transId
 * @param success   必选，只有接口返回成功，才可以进入提交页面
 * @param fail      可选，失败回调
 */
export let getSubmitKey = (url = "",
						   opId = "",
						   transId = "",
						   success,
						   fail) => {
	simpleRequest(
		url,
		opId,
		transId,
		{},
		function (response) {
			__bus__.global.submitKey = response.rspData[SUBMIT_KEY];
			success && success();
		},
		fail,
		{loadingViewFullScreenFlag: true}
	);
};
/**
 * 上传图片
 * @param opt
 */
export const uploadImage = opt => {
	_requestByNative(opt, "uploadImage");
};

/**
 * 长安报文加密用
 * @param json
 * @param callback
 */
export const encryptRequestJson = (json, callback) => {
	callHandler('encryptRequestJson', {
		data: json,
		success: callback
	});
};

/**
 * AES报文加密
 * @param json
 * @param callback
 */
export const encryptAESJson = (json, callback) => {
	callHandler('encryptAESJson', {
		data: json,
		success: callback
	});
};

/////////////////////////////////////////////////////////////////
function _request(object) {
	let _object = _requestParameterInterceptor(object);
	if (isInApp()) {
		_requestByNative(_object);
	} else {
		_requestByAjax(_object);
	}
}

/////////////////////////////////////////////////////////////////
/**网络请求拦截器*/
/**
 * 请求参数拦截，自定义处理
 * @param object
 * @returns {*}
 * @private
 */
function _requestParameterInterceptor(object) {
	if (requestInterceptor.beforeCallback) {
		return requestInterceptor.beforeCallback(object) || {};
	}
	return object;
}

/**
 * 网络请求，成功回调拦截器
 * @param response 网络请求成功回调报文
 * @param nextCallback 继续操作回调函数
 * @returns {*}
 * @private
 */
function _requestSuccessInterceptor(response, nextCallback) {
	if (requestInterceptor.afterSuccess) {
		let flag = requestInterceptor.afterSuccess(response);
		if (flag) {
			return nextCallback && nextCallback();
		}
	}
	return nextCallback && nextCallback();
}

/**
 * 网络请求，失败回调拦截器
 * @param response 网络请求失败回调报文
 * @param nextCallback
 * @returns {*}
 * @private
 */
function _requestFailInterceptor(response, nextCallback) {
	if (requestInterceptor.afterFail) {
		let flag = requestInterceptor.afterFail(response);
		if (flag) {
			return nextCallback && nextCallback();
		}
	}
	return nextCallback && nextCallback();
}

/////////////////////////////////////////////////////////////////
function _requestByNative(object, methodName = "request") {
	if (isString(object.url)) {
		if (!/^(http|https):\/\/.+/g.test(object.url)) {
			//不是完整url路径
			let baseUrl = __bus__.global.device.baseUrl || "";
			baseUrl = baseUrl.replace(/\/$/, "") + "/";
			object.url = baseUrl + object.url;
		}
	}
	let defaultSetting = {
		showLoadingViewFlag: true,
		loadingViewFullScreenFlag: false
	};
	let jsonData = object.data;
	jsonData.reqHead.submitKey = __bus__.global.submitKey || ""; //复制submitKey
	callHandler(methodName, {
		url: object.url,
		setting: extend(defaultSetting, object.setting),
		data: jsonData,
		success: function (response) {
			if (response && response.rspHead) {
				if (response.rspHead.rspCd === "S000A000") {
					_requestSuccessInterceptor(
						response,
						() => object.success && object.success(response)
					);
				} else {
					_requestFailInterceptor(response, () => {
						let message = response.rspHead.rspCd + ":" + response.rspHead.msg;
						return object.fail && object.fail(message, response);
					});
				}
			}else {
				log(`网络请求参数错误，response = ${JSON.stringify(response)}`);
			}
		},
		fail: function (err) {
			_requestFailInterceptor(err, () => {
				let message = '';
				if(err && err.rspHead){
					message = err.rspHead.rspCd + ":" + err.rspHead.msg
				}
				return object.fail && object.fail(message, err);
			});
		}
	});
}

function _requestByAjax(object) {
	_ajax({
		url: object.url,
		data: {jsonData: JSON.stringify(object.data)},
		success: function (result) {
			let response;
			try {
				response = JSON.parse(result);
			} catch (err) {
				response = {};
			}
			if (response.rspHead.rspCd === "S000A000") {
				_requestSuccessInterceptor(
					response,
					() => object.success && object.success(response)
				);
			} else {
				_requestFailInterceptor(response, () => {
					let message = response.rspHead.rspCd + ":" + response.rspHead.msg;
					return object.fail && object.fail(message, response);
				});
			}
		},
		fail: function (err) {
			_requestFailInterceptor(err, () => {
				let message = err.rspHead.rspCd + ":" + err.rspHead.msg;
				return object.fail && object.fail(message, err);
			});
		}
	});
}

/* 封装ajax函数
 * @param {string}opt.type http连接的方式，包括POST和GET两种方式
 * @param {string}opt.url 发送请求的url
 * @param {boolean}opt.async 是否为异步请求，true为异步的，false为同步的
 * @param {object}opt.data 发送的参数，格式为对象类型
 * @param {function}opt.success ajax发送并接收成功调用的回调函数
 */
function _ajax(opt) {
	opt = opt || {};
	opt.method = "POST"; //写死POST
	opt.url = opt.url || "";
	opt.async = opt.async || true;
	opt.data = opt.data || null;
	opt.success = opt.success || function () {
	};
	opt.fail = opt.fail || function () {
	};
	let xmlHttp = null;
	if (XMLHttpRequest) {
		xmlHttp = new XMLHttpRequest();
	} else {
		xmlHttp = new ActiveXObject("Microsoft.XMLHTTP");
	}
	let params = [];
	for (let key in opt.data) {
		params.push(key + "=" + opt.data[key]);
	}
	let postData = params.join("&");
	if (opt.method.toUpperCase() === "POST") {
		xmlHttp.open(opt.method, opt.url, opt.async);
		xmlHttp.setRequestHeader(
			"Content-Type",
			"application/x-www-form-urlencoded;charset=utf-8"
		);
		xmlHttp.send(postData);
	} else if (opt.method.toUpperCase() === "GET") {
		xmlHttp.open(opt.method, opt.url + "?" + postData, opt.async);
		xmlHttp.send(null);
	}
	xmlHttp.onreadystatechange = function () {
		if (xmlHttp.readyState === 4) {
			if(xmlHttp.status === 200){
				return opt.success(xmlHttp.responseText);
			}else {
				opt.fail({rspHead: {rspCd: xmlHttp.status , msg: xmlHttp.statusText}, rspData: {}});
			}
		}
	};
}
