import {__type__ } from './__handle__/__handle_utils__';
import __bus__ from "./__handle__/__bus__";
import {FUNCTION, STRING, ARRAY, BOOLEAN, OBJECT, NAN, NUMBER} from "./const";
import appInfo from './appinfo';

/**
 * 是否是安卓系统浏览器环境
 * @param arg 浏览器环境则参数不生效，test:unit环境允许传入参数，使用模拟的ua, 下同
 * @returns {boolean}
 */
export const isAndroid = () => /android/i.test(__bus__.userAgent);

/**
 * 是否是iOS系统浏览器环境
 * @param arg
 * @returns {boolean}
 */
export const isIOS =  () => /iphone|ipad/i.test(__bus__.userAgent);

/**
 * 是否是微信浏览器环境
 * @param arg
 * @returns {boolean}
 */
export const isWeChat = () => /MicroMessenger/i.test(__bus__.userAgent);

/**
 * 是否在在APP webview环境，而不是在browser
 * @param arg
 * @returns {boolean}
 */
export const isInApp = () => /@sdkVer/i.test(__bus__.userAgent);

/**
 * 是否是方法函数
 * @param fn
 * @returns {boolean}
 */
export const isFunction = fn => FUNCTION === __type__(fn);

/**
 * 是否是字符串
 * @param arg
 * @returns {boolean}
 */
export const isString = arg => STRING === __type__(arg);

/**
 * 是否是数组
 * @param arg
 * @returns {boolean}
 */
export const isArray = arg => ARRAY === __type__(arg);

/**
 * 是否是boolean值
 * @param arg
 * @returns {boolean}
 */
export const isBoolean = arg => BOOLEAN === __type__(arg);

/**
 * 是否是对象，这个方法对array判断不为object
 * @param arg
 * @returns {boolean}
 */
export const isObject = arg => OBJECT === __type__(arg);

/**
 * 不能转成number的字符串
 * @param arg
 * @returns {boolean}
 */
export const isNaN = arg => isNaN(arg);

/**
 * 是否是number类型
 * @param arg
 * @returns {boolean}
 */
export const isNumber = arg => NUMBER === __type__(arg);

/**
 * 是否是undefined
 * @param arg
 * @returns {boolean}
 */
export const isUndefined = arg =>  arg === void 0;

/**
 * 是否是空对象
 * @param obj
 * @returns {boolean}
 */
export const isEmptyObject = (obj) => {
	for (let v in obj) {
		return false;
	}
	return true;
};

/**
 * 对象扩展
 * @method extend
 * @param  {Object} obj  原始对象
 * @param  {Object} args 多个继承对象
 * @return {Object}      扩展后对象
 */
export function extend(obj) {
	let source;
	let prop;
	if (!isObject(obj)) {
		return obj;
	}

	for (
		var _len5 = arguments.length,
			args = Array(_len5 > 1 ? _len5 - 1 : 0),
			_key5 = 1;
		_key5 < _len5;
		_key5++
	) {
		args[_key5 - 1] = arguments[_key5];
	}

	for (let i = 0, length = args.length; i < length; i++) {
		source = args[i];
		for (prop in source) {
			if (Object.prototype.hasOwnProperty.call(source, prop)) {
				obj[prop] = source[prop];
			}
		}
	}
	return obj;
}

/**
 * 版本比较函数
 * @param targetVer1
 * @param targetVer2
 * @return {number}
 *          1: targetVer1 > targetVer2
 *         -1: targetVer1 < targetVer2
 *          0: targetVer1 = targetVer2
 */
export function compareVersion(targetVer1 = "", targetVer2 = "") {
	let reg = /^\d+(\.\d+){0,2}$/;
	if (!reg.test(targetVer1)) {
		return 0;
	}
	if (!reg.test(targetVer2)) {
		return 0;
	}
	let _targetVer1 = targetVer1.split(".");
	let _targetVer2 = targetVer2.split(".");
	for (let i = 0, n1, n2; i < _targetVer1.length; i++) {
		n1 = parseInt(_targetVer1[i], 10) || 0;
		n2 = parseInt(_targetVer2[i], 10) || 0;
		if (n1 < n2) {
			return -1;
		}
		if (n1 > n2) {
			return 1;
		}
	}
	return 0;
}

/**
 * SDK版本比较方法
 * @param targetVersion 目标版本和当前的sdk的引擎版本比较
 * @returns {number}
 *          1：表目标比当前版本小；
 *         -1：表目标比当前版本大；
 *          0：表目标相等；
 */
export function compareSdkVersion(targetVersion) {
	return compareVersion(appInfo.engineVer, targetVersion);
}

/**
 * sdk版本不低于指定版本
 * @param targetVer 指定版本
 * @return {boolean}
 *          true：sdkVer >= targetVer
 *         false: sdkVer < targetVer
 */
export function sdkVerAsLastWith(targetVer) {
	return compareVersion(appInfo.engineVer, targetVer) >= 0;
}



/**
 * 解析URL
 * @param url 带解析的url
 * @return {{source: string, protocol: string, host: string, port: string, query: string, params, file: string | *, hash: string, path: string, relative: string | *, segments: string[]}}
 */
export const parseURL = (url = location.href) => {
	let a = document.createElement("a");
	a.href = url;
	return {
		source: url,
		protocol: a.protocol.replace(":", ""),
		host: a.hostname,
		port: a.port,
		query: a.search,
		param: (function() {
			let result = {},
				bool = { true: true, false: false },
				seg = a.search.replace(/^\?/, "").split("&"),
				len = seg.length;

			for (let i = 0; i < len; i++) {
				if (!seg[i]) {
					continue;
				}
				let kv = seg[i].split("=");
				kv[1] = decodeURIComponent(kv[1]);
				kv[1] = isUndefined(bool[kv[1]]) ? kv[1] : bool[kv[1]];
				result[kv[0]] = kv[1];
			}
			return result;
		})(),
		file: (a.pathname.match(/\/([^\/?#]+)$/i) || [, ""])[1],
		hash: a.hash.replace("#", ""),
		path: a.pathname.replace(/^([^\/])/, "/$1"),
		relative: (a.href.match(/tps?:\/\/[^\/]+(.+)/) || [, ""])[1],
		segments: a.pathname.replace(/^\//, "").split("/")
	};
};

/**
 * json转成queryString字符串化
 * @param json      带转换的对象
 * @return {string} queryString 字符串化
 */
export const queryStringify = (json = {}) => {
	let array = [];
	for (let key in json) {
		let value = json[key];
		// value = JSON.stringify(value);
		array.push(encodeURIComponent(key) + "=" + encodeURIComponent(value));
	}
	return array.join("&");
};

/**
 * 构造带参的完整 url
 * @method buildUrl
 * @param  {String}   url    原始 url，可能已经有 queryString
 * @param  {Object}   params url 参数对象
 * @return {String}          拼接好的带参 url
 */
export const buildUrl = (url, params) => {
	let qs = params;
	let urls = url.split("#");
	if (isObject(params)) {
		qs = queryStringify(params);
	}
	if (!/\?/.test(url)) {
		qs = "?" + qs;
	} else if (!/&$/.test(url) && !/\?$/.test(url)) {
		qs = "&" + qs;
	}
	let returnUrl = url + qs;
	if (urls.length === 2) {
		let arr = urls[0].split("?");
		if (arr.length === 2) {
			qs = "?" + arr[1] + qs;
		}
		returnUrl = urls[0] + qs + "#" + urls[1];
	}
	return returnUrl;
};

